#include <stdio.h>
#include <windows.h>
#include "..\common\CSerial.h"
#include "..\common\CSocket.h"
#include "..\common\LE9x0Command.h"

int main()
{
	int ret = 0;

	// -------------------------------------------------
	// Select the communication method with LE-910R.
	// For the USB interface, open the virtual COM port.
	// For the WiFi interfaces, open a TCP socket.
	// -------------------------------------------------
#if 1
	// Open COM port (COM1)
	CSerial *comm = new CSerial();
	int comPort = 1;
	if( !comm->Open( (void*)&comPort ) )
	{
		ret = -1;
	}
#else
	// Open TCP socket
	// (IP Address = 192.168.0.10, Control port number = 10101)
	CSocket *comm = new CSocket();
	CSocket::CONNECTION connetion = { "192.168.0.10", 10101 };
	if( !comm->Open( (void*)&connetion ))
	{
		ret = -1;
	}
#endif

	if( ret < 0 )
	{
		printf( "Interface open error.\n" );
		goto error_exit;
	}

	// ------------------
	// Connect to LE-910R
	// ------------------
	FRAME Res;
	LE9x0Command *cmd = new LE9x0Command( comm );

	// Invalid connection maintenance command transmission from LE-910R
	cmd->TransmitCommandWait( CMD_CONNECT, 0x20, 0, NULL, &Res );
	if( (Res.res != RES_OK) && (Res.res != RES_ALREADY_CONNECTED) )
	{
		printf( "Connection error. [0x%02X]\n", Res.res );
		ret = -1;
		goto error_comm;
	}
	printf( "Connection OK.\n" );

	// -----------------
	// Setup for LE-910R
	// ------------------
	// Thermocouple measurement setting
	// Ch = AI5, Type = K
	// Option = Internal compensation & Enable disconnection detection(output=0x800000)
	BYTE tc_cfg[3] = { 0x10, 0, 0x03 };
	cmd->TransmitCommandWait( CMD_TEMP_SET, 0x00, sizeof(tc_cfg), tc_cfg, &Res );
	if( Res.res != RES_OK )
	{
		printf( "Thermocouple measurement setting error. [0x%02X]\n", Res.res );
		goto error_cmd;
	}

	// AD converter conversion speed setting ( SPS=10sps )
	BYTE sps = 0;
	cmd->TransmitCommandWait( CMD_AD_SPS_SET, 0x00, 1, &sps, &Res );
	if( Res.res != RES_OK )
	{
		printf( "AD converter conversion speed setting error. [0x%02X]\n", Res.res );
		goto error_cmd;
	}

	// Analog input range setting
	// AI1 = 100mV, AI2 = 10V, AI3 = 30V, AI4 = 4-20mA, AI5 = Thermocouple
	RANGE range_cfg[5] = { RANGE_100mV, RANGE_10V, RANGE_30V, RANGE_I_250ohm, RANGE_TC };
	for( int i = 0; i < 5; i++ )
	{
		BYTE range[2];
		range[0] = 0x01 << i;		// Ch mask
		range[1] = range_cfg[i];	// Range
		cmd->TransmitCommandWait( CMD_AD_RANGE_SET, 0x00, sizeof(range), range, &Res );
		if( Res.res != RES_OK )
		{
			printf( "Analog input range setting error.(AI%d) [0x%02X]\n", i+1, Res.res );
			goto error_cmd;
		}
	}

	// -------------------------------
	// Acquire and convert analog data
	// -------------------------------
	// Acquires and converts analog data of all channels about every second.
	// The measurement is performed 10 times.
	for( int i = 0; i < 10; i++ )
	{
		printf( "No.%2d", i+1 );
		for( int ch = 0; ch < 5; ch++ )
		{
			// Analog data acquisition
			cmd->TransmitCommandWait( CMD_AD_VALUE_GET, 0x00, 1, (BYTE*)&ch, &Res );
			if( Res.res != RES_OK )
			{
				printf( "\nAnalog data acquisition error.(AI%d) [0x%02X]\n", i+1, Res.res );
				break;
			}


			// Conversion of analog data
			DWORD ad;
			ad  = (DWORD)Res.data[2] << 16;
			ad |= (DWORD)Res.data[3] << 8;
			ad |= (DWORD)Res.data[4];
			float val = cmd->ConvertAnalogDataFromLE910( range_cfg[ch], ad );

			printf( ",AI%d:", ch+1 );
			switch( Res.data[1] )
			{
			case RANGE_100mV:
				printf( "%.3f[mV]", val );
				break;
			case RANGE_1V:
				printf( "%.5f[V]", val );
				break;
			case RANGE_10V:
			case RANGE_30V:
				printf( "%.4f[V]", val );
				break;
			case RANGE_I_250ohm:
			case RANGE_I_50ohm:
				printf( "%.4f[mA]", val );
				break;
			case RANGE_TC:
				if( (val == 0x800000) || (val == 0x7FFFFF) )
					printf( "BO" );
				else
					printf( "%.1f[degC]", val );
				break;
			}
		}
		printf( "\n" );
		Sleep( 1000 );
	}

error_cmd:
	// ----------------------
	// Disonnect from LE-910R
	// ----------------------
	cmd->TransmitCommandWait( CMD_DISCONNECT, 0x00, 0, NULL, &Res );
	if( Res.res != RES_OK )
	{
		printf( "Disconnection error. [0x%02X]\n", Res.res );
		ret = -1;
		goto error_comm;
	}
	printf( "Disconnection OK.\n" );

error_comm:
	comm->Close();
	delete cmd;

error_exit:
	delete comm;

	return ret;
}
